package dashboard

import (
	"gorm.io/driver/mysql"
	"gorm.io/gorm"
	"gorm.io/gorm/logger"

	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/ctx"
	"shylinux.com/x/icebergs/base/mdb"
	"shylinux.com/x/icebergs/base/web"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/operation/src/dashboard/model"
)

type summary struct {
	Table
	client     client
	userClient userClient
	create     string `name:"create title* space index* score" role:"leader"`
	modify     string `name:"modify title* space index* score" role:"leader"`
	remove     string `name:"remove" role:"leader"`
	scan       string `name:"scan" role:"leader"`
}

func (s summary) Create(m *ice.Message, arg ...string) {
	s.Table.Create(m, kit.Simple(arg, m.OptionSimple(model.USER_UID, model.CLIENT_UID))...)
	s.RecordEventWithName(m, "")
}
func (s summary) Remove(m *ice.Message, arg ...string) {
	s.Table.Delete(m, m.OptionSimple(model.CLIENT_UID, model.UID)...)
}
func (s summary) Modify(m *ice.Message, arg ...string) {
	s.Table.Update(m, kit.Dict(arg), m.OptionSimple(model.CLIENT_UID, model.UID)...)
}
func (s summary) Scan(m *ice.Message, arg ...string) {
	if false {
		s.List(m, arg...).Table(func(value ice.Maps) {
			msg := m.Cmd(web.SPACE, value[web.SPACE], value[ctx.INDEX], s.Select, kit.Dict(mdb.FIELDS, kit.Select("count(*)", value[model.FIELDS])))
			s.Update(m, kit.Dict(model.VALUE, msg.Append(msg.Append(ice.MSG_APPEND))), model.UID, value[model.UID])
		})
		return
	}
	s.Open(m, func(db *gorm.DB) {
		s.List(m, arg...).Table(func(value ice.Maps) {
			db = db.Table(s.TableName(kit.Join(kit.Slice(kit.Split(value[ctx.INDEX], "."), -2), ".")))
			msg := s.Rows(m.Spawn(), db.Select(kit.Select("count(*)", value[model.FIELDS])))
			s.Update(m, kit.Dict(model.VALUE, msg.Append(msg.Append(ice.MSG_APPEND))), model.UID, value[model.UID])
		})
	})
}
func (s summary) List(m *ice.Message, arg ...string) *ice.Message {
	s.Orders(m, model.SCORE, s.Desc(model.CREATED_AT))
	s.Select(m, model.CLIENT_UID, arg[0]).Action(s.Create, s.Scan).PushAction(s.Modify, s.Remove).Display("").DisplayCSS("")
	if m.Length() == 0 {
		m.EchoInfoButton("")
	}
	return m
}

func init() { ice.TeamCtxCmd(summary{}) }

func (s summary) Open(m *ice.Message, cb func(*gorm.DB)) {
	msg := m.Cmd(s.client, s.Select, model.UID, m.Option(model.CLIENT_UID))
	dsn := kit.Format("%s:%s@tcp(%s:%s)/%s?charset=utf8mb4&parseTime=True", msg.Append(model.USERNAME), msg.Append(model.PASSWORD), msg.Append(model.HOST), msg.Append(model.PORT), msg.Append(model.DATABASE))
	if db, err := gorm.Open(mysql.Open(dsn), &gorm.Config{Logger: logger.Default.LogMode(logger.Info)}); !m.Warn(err) {
		cb(db)
	}
}
